package cn.iocoder.yudao.module.bpm.service.oa;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.invoice.BpmOAInvoiceCreateReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAInvoiceDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAInvoiceMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_INVOICE_NOT_EXISTS;

/**
 * OA 开票申请 Service 实现类
 *
 * @author 符溶馨

 */
@Service
@Validated
public class BpmOAInvoiceServiceImpl extends BpmOABaseService implements BpmOAInvoiceService{

    /**
     * OA 开票申请对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_invoice_2";

    @Resource
    private BpmOAInvoiceMapper invoiceMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createInvoice(Long userId, BpmOAInvoiceCreateReqVO createReqVO) {

        //插入OA 开票申请
        BpmOAInvoiceDO invoice = BeanUtils.toBean(createReqVO, BpmOAInvoiceDO.class)
                .setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        invoiceMapper.insert(invoice) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(invoice.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 开票申请单中
        invoiceMapper.updateById(new BpmOAInvoiceDO().setId(invoice.getId()).setProcessInstanceId(processInstanceId));

        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return invoice.getId();
    }

    @Override
    public void updateInvoiceResult(Long id, Integer result) {

        validateLeaveExists(id);
        invoiceMapper.updateById(new BpmOAInvoiceDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (invoiceMapper.selectById(id) == null) {
            throw exception(OA_INVOICE_NOT_EXISTS);
        }
    }

    @Override
    public BpmOAInvoiceDO getInvoice(Long id) {

        return invoiceMapper.selectById(id);
    }

    @Override
    public BpmOAInvoiceDO getByProcessInstanceId(String processInstanceId) {

        List<BpmOAInvoiceDO> bpmOAInvoiceDOS = invoiceMapper.selectList(BpmOAInvoiceDO::getProcessInstanceId, processInstanceId);
        if (CollectionUtil.isEmpty(bpmOAInvoiceDOS)) {
            return null;
        }
        return bpmOAInvoiceDOS.get(0);
    }
}
