package cn.iocoder.yudao.module.crm.service.crmachievement;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.enums.ShopCommonEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.crm.controller.admin.crmachievement.vo.*;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmachievement.CrmAchievementDO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmcontract.CrmContractDO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmcontractreceivables.CrmContractReceivablesDO;
import cn.iocoder.yudao.module.crm.dal.mysql.crmachievement.CrmAchievementMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmcontract.CrmContractMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmcontractreceivables.CrmContractReceivablesMapper;
import cn.iocoder.yudao.module.hrm.enums.ContractStatusEnum;
import cn.iocoder.yudao.module.hrm.enums.FlowStepEnum;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptApiDTO;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptApiVO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserApiDTO;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserApiVO;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserPageApiDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.hrm.enums.ErrorCodeConstants.ACHIEVEMENT_NOT_EXISTS;


/**
 * 业绩目标 Service 实现类
 *
 * @author yshop
 */
@Service
@Validated
public class CrmAchievementServiceImpl implements CrmAchievementService {

    @Resource
    private CrmAchievementMapper achievementMapper;
    @Resource
    private DeptApi deptService;
    @Resource
    private CrmContractMapper contractMapper;
    @Resource
    private CrmContractReceivablesMapper contractReceivablesMapper;
    @Resource
    private AdminUserApi adminUserApi;


    @Override
    public void createAchievement(DeptAchieveSaveVO createReqVO) {

        if (createReqVO.getId() == 0) {
            //批量
            if (FlowStepEnum.TYPE_3.getValue().equals(createReqVO.getAchievementRespVO().getType())) {
                List<DeptApiVO> list = deptService.getDeptList(
                        new DeptApiDTO().setStatus(CommonStatusEnum.ENABLE.getStatus())).getCheckedData();
                list.forEach(dept -> {
                    createReqVO.setId(dept.getId());
                    createReqVO.setName(dept.getName());
                    saveAchievement(createReqVO);
                });
            } else {
                List<AdminUserApiVO> list = adminUserApi.getUserList(new AdminUserApiDTO().setStatus(CommonStatusEnum.ENABLE.getStatus())).getCheckedData();
                list.forEach(user -> {
                    createReqVO.setId(user.getId());
                    createReqVO.setName(user.getNickname());
                    saveAchievement(createReqVO);
                });
            }
        } else {
            //单个操作
            if (FlowStepEnum.TYPE_3.getValue().equals(createReqVO.getAchievementRespVO().getType())) {
                createReqVO.setName(createReqVO.getName());
            } else {
                createReqVO.setName(createReqVO.getNickname());
            }
            saveAchievement(createReqVO);
        }


    }

    /**
     * 添加或者更新
     *
     * @param createReqVO
     */
    private void saveAchievement(DeptAchieveSaveVO createReqVO) {
        // 插入
        CrmAchievementDO achievement = BeanUtils.toBean(createReqVO.getAchievementRespVO(), CrmAchievementDO.class);
        achievement.setTypeId(createReqVO.getId());
        achievement.setName(createReqVO.getName());
        LambdaQueryWrapper<CrmAchievementDO> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(CrmAchievementDO::getType, achievement.getType())
                .eq(CrmAchievementDO::getTypeId, achievement.getTypeId())
                .eq(CrmAchievementDO::getConfig, achievement.getConfig())
                .eq(CrmAchievementDO::getYear, achievement.getYear());
        CrmAchievementDO achievementOld = achievementMapper.selectOne(wrapper);
        if (achievementOld == null) {
            achievementMapper.insert(achievement);
        } else {
            achievement.setId(achievementOld.getId());
            achievementMapper.updateById(achievement);
        }

        //更新年目标
        List<BigDecimal> bigDecimals = Arrays.asList(
                achievement.getJanuary(), achievement.getFebruary(), achievement.getMarch(), achievement.getApril(),
                achievement.getMay(), achievement.getJune(), achievement.getJuly(), achievement.getAugust(),
                achievement.getSeptember(), achievement.getOctober(), achievement.getNovember(), achievement.getDecember()
        );
        BigDecimal yearTarget = bigDecimals.stream()
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        CrmAchievementDO achievementDO = CrmAchievementDO.builder().yeartarget(yearTarget).id(achievement.getId()).build();
        achievementMapper.updateById(achievementDO);
    }

    @Override
    public void updateAchievement(CrmAchievementSaveReqVO updateReqVO) {
        // 校验存在
        validateAchievementExists(updateReqVO.getId());
        // 更新
        CrmAchievementDO updateObj = BeanUtils.toBean(updateReqVO, CrmAchievementDO.class);
        achievementMapper.updateById(updateObj);
    }

    @Override
    public void deleteAchievement(Long id) {
        // 校验存在
        validateAchievementExists(id);
        // 删除
        achievementMapper.deleteById(id);
    }

    private void validateAchievementExists(Long id) {
        if (achievementMapper.selectById(id) == null) {
            throw exception(ACHIEVEMENT_NOT_EXISTS);
        }
    }

    @Override
    public CrmAchievementDO getAchievement(Long id) {
        return achievementMapper.selectById(id);
    }

    @Override
    public PageResult<UserAchieveRespVO> getAchievementPage(CrmAchievementPageReqVO pageReqVO) {
        AdminUserPageApiDTO userPageReqVO = new AdminUserPageApiDTO();
        userPageReqVO.setPageNo(pageReqVO.getPageNo());
        userPageReqVO.setPageSize(pageReqVO.getPageSize());
        // -- 这里固定查询销售的
        userPageReqVO.setRoleCode("sale");
        PageResult<AdminUserApiVO> pageResult = adminUserApi.getUserBringDeptPage(userPageReqVO).getCheckedData();
        PageResult<UserAchieveRespVO> pageResult1 = BeanUtils.toBean(pageResult, UserAchieveRespVO.class);
        List<Long> typeIds = new ArrayList<>();
        for (UserAchieveRespVO vo : pageResult1.getList()) {
            typeIds.add(vo.getId());
        }
        typeIds = typeIds.stream().distinct().collect(Collectors.toList());
        List<CrmAchievementDO> crmAchievementDOS = achievementMapper.selectList(new LambdaQueryWrapper<CrmAchievementDO>()
                .eq(CrmAchievementDO::getType, FlowStepEnum.TYPE_2.getValue())
                .eq(CrmAchievementDO::getConfig, pageReqVO.getConfig())
                .in(CrmAchievementDO::getTypeId, typeIds)
                .eq(CrmAchievementDO::getYear, pageReqVO.getYear())
        );

        Map<Long, List<CrmAchievementDO>> map = crmAchievementDOS.stream().collect(Collectors.groupingBy(CrmAchievementDO::getTypeId));

        pageResult1.getList().forEach(v -> {
            List<CrmAchievementDO> doList = map.get(v.getId());
            CrmAchievementDO crmAchievementDO = null;
            if (CollUtil.isNotEmpty(doList)) {
                crmAchievementDO = doList.get(0);
            }
            if (crmAchievementDO == null) {
                crmAchievementDO = CrmAchievementDO.builder()
                        .january(BigDecimal.ZERO).february(BigDecimal.ZERO).march(BigDecimal.ZERO)
                        .april(BigDecimal.ZERO).may(BigDecimal.ZERO).june(BigDecimal.ZERO)
                        .july(BigDecimal.ZERO).august(BigDecimal.ZERO).september(BigDecimal.ZERO)
                        .october(BigDecimal.ZERO).november(BigDecimal.ZERO).december(BigDecimal.ZERO)
                        .yeartarget(BigDecimal.ZERO)
                        .build();
            }
            v.setAchievementRespVO(BeanUtils.toBean(crmAchievementDO, CrmAchievementRespVO.class));
        });
        return pageResult1;
    }

    @Override
    public List<DeptAchieveRespVO> getDeptAchieve(Integer type, Integer year) {
        List<DeptApiVO> list = deptService.getDeptList(
                new DeptApiDTO().setStatus(CommonStatusEnum.ENABLE.getStatus())).getCheckedData();
        List<DeptAchieveRespVO> achieveRespVOS = BeanUtils.toBean(list, DeptAchieveRespVO.class);
        List<Long> typeIds = new ArrayList<>();
        for (DeptAchieveRespVO vo : achieveRespVOS) {
            typeIds.add(vo.getId());
        }
        typeIds = typeIds.stream().distinct().collect(Collectors.toList());
        List<CrmAchievementDO> crmAchievementDOS = achievementMapper.selectList(new LambdaQueryWrapper<CrmAchievementDO>()
                .eq(CrmAchievementDO::getType, FlowStepEnum.TYPE_3.getValue())
                .eq(CrmAchievementDO::getConfig, type)
                .in(CrmAchievementDO::getTypeId, typeIds)
                .eq(CrmAchievementDO::getYear, year)
        );
        Map<Long, List<CrmAchievementDO>> map = crmAchievementDOS.stream().collect(Collectors.groupingBy(CrmAchievementDO::getTypeId));
        achieveRespVOS.forEach(vo -> {
            List<CrmAchievementDO> doList = map.get(vo.getId());
            CrmAchievementDO crmAchievementDO = null;
            if (CollUtil.isNotEmpty(doList)) {
                crmAchievementDO = doList.get(0);
            }
            if (crmAchievementDO == null) {
                crmAchievementDO = CrmAchievementDO.builder()
                        .january(BigDecimal.ZERO).february(BigDecimal.ZERO).march(BigDecimal.ZERO)
                        .april(BigDecimal.ZERO).may(BigDecimal.ZERO).june(BigDecimal.ZERO)
                        .july(BigDecimal.ZERO).august(BigDecimal.ZERO).september(BigDecimal.ZERO)
                        .october(BigDecimal.ZERO).november(BigDecimal.ZERO).december(BigDecimal.ZERO)
                        .yeartarget(BigDecimal.ZERO)
                        .build();
            }
            vo.setAchievementRespVO(BeanUtils.toBean(crmAchievementDO, CrmAchievementRespVO.class));
        });
        return achieveRespVOS;
    }

    @Override
    public AchieveCountRespVO getCount(Integer type, Integer year, Integer month) {
        List<Long> userIds = new ArrayList<>();
        Long typeId = SecurityFrameworkUtils.getLoginUserId();
        if (FlowStepEnum.TYPE_2.getValue().equals(type)) {
            userIds.add(typeId);
        } else {
            //部门
            String deptId = Objects.requireNonNull(SecurityFrameworkUtils.getLoginUser()).getInfo().get(SecurityFrameworkUtils.INFO_KEY_DEPT_ID);
            typeId = deptId == null ? null : Long.valueOf(deptId);
            List<AdminUserApiVO> adminUserDOS = new ArrayList<>();
            if (typeId != null) {
                adminUserDOS = adminUserApi.getUserList(new AdminUserApiDTO().setDeptId(typeId).setNextDept(1)).getCheckedData();
            }
            userIds = adminUserDOS.stream().map(AdminUserApiVO::getId).collect(Collectors.toList());
        }
        Date starTime = new Date();
        Date endTime = new Date();
        Calendar calendar = Calendar.getInstance();
        if (month == 0) {
            //全年
            calendar.set(year, 1, 1);
            starTime = DateUtil.beginOfYear(calendar.getTime());
            endTime = DateUtil.endOfYear(calendar.getTime());
        } else {
            //某个月
            calendar.set(year, month - 1, 1);
            starTime = DateUtil.beginOfMonth(calendar.getTime());
            endTime = DateUtil.endOfMonth(calendar.getTime());
        }

        //合同目标
        CrmAchievementDO crmAchievementDO = achievementMapper.selectOne(new LambdaQueryWrapperX<CrmAchievementDO>()
                .eq(CrmAchievementDO::getType, type)
                .eq(CrmAchievementDO::getTypeId, typeId)
                .eqIfPresent(CrmAchievementDO::getConfig, ShopCommonEnum.ACH_1.getValue())
                .eqIfPresent(CrmAchievementDO::getYear, year));
        //回款目标
        CrmAchievementDO crmAchievementDO2 = achievementMapper.selectOne(new LambdaQueryWrapperX<CrmAchievementDO>()
                .eq(CrmAchievementDO::getType, type)
                .eq(CrmAchievementDO::getTypeId, typeId)
                .eqIfPresent(CrmAchievementDO::getConfig, ShopCommonEnum.ACH_2.getValue())
                .eqIfPresent(CrmAchievementDO::getYear, year));


        List<CrmContractDO> crmContractDOS = contractMapper.selectList(new LambdaQueryWrapper<CrmContractDO>()
                .in(CrmContractDO::getOwnerUserId, userIds)
                .eq(CrmContractDO::getCheckStatus, ContractStatusEnum.STATUS_2.getValue())
                .between(CrmContractDO::getOrderTime, starTime, endTime));
        BigDecimal contractSuccessMoney = BigDecimal.ZERO;
        if (!crmContractDOS.isEmpty()) {
            contractSuccessMoney = crmContractDOS
                    .stream()
                    .map(CrmContractDO::getMoney)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        List<CrmContractReceivablesDO> crmContractReceivablesDOS = contractReceivablesMapper
                .selectList(new LambdaQueryWrapper<CrmContractReceivablesDO>()
                        .in(CrmContractReceivablesDO::getOwnerUserId, userIds)
                        .eq(CrmContractReceivablesDO::getCheckStatus, ContractStatusEnum.STATUS_2.getValue())
                        .between(CrmContractReceivablesDO::getReturnTime, starTime, endTime));
        BigDecimal receivablesSuccessMoney = BigDecimal.ZERO;
        if (!crmContractReceivablesDOS.isEmpty()) {
            receivablesSuccessMoney = crmContractReceivablesDOS
                    .stream()
                    .map(CrmContractReceivablesDO::getMoney)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }

        BigDecimal contractMoney = BigDecimal.ZERO;
        BigDecimal receivablesMoney = BigDecimal.ZERO;
        String contractPer = "0";
        String receivablesPer = "0";
        switch (month) {
            case 0: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getYeartarget();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getYeartarget();
                }
            }
            case 1: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getJanuary();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getJanuary();
                }
            }
            case 2: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getFebruary();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getFebruary();
                }
            }
            case 3: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getMarch();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getMarch();
                }
            }
            case 4: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getApril();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getApril();
                }
            }
            case 5: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getMay();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getMay();
                }
            }
            case 6: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getJune();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getJune();
                }
            }
            case 7: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getJuly();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getJuly();
                }
            }
            case 8: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getAugust();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getAugust();
                }
            }
            case 9: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getSeptember();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getSeptember();
                }
            }
            case 10: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getOctober();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getOctober();
                }
            }
            case 11: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getNovember();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getNovember();
                }
            }
            case 12: {
                if (crmAchievementDO != null) {
                    contractMoney = crmAchievementDO.getDecember();
                }
                if (crmAchievementDO2 != null) {
                    receivablesMoney = crmAchievementDO2.getDecember();
                }
            }


        }
        if (contractMoney.compareTo(BigDecimal.ZERO) > 0) {
            contractPer = NumberUtil.round(NumberUtil.div(contractSuccessMoney, contractMoney), 2)
                    .multiply(new BigDecimal("100")).toString();
        }
        if (receivablesMoney.compareTo(BigDecimal.ZERO) > 0) {
            receivablesPer = NumberUtil.round(NumberUtil.div(receivablesSuccessMoney, receivablesMoney), 2)
                    .multiply(new BigDecimal("100")).toString();
        }

        return AchieveCountRespVO.builder()
                .contractMoney(contractMoney)
                .contractSuccessMoney(contractSuccessMoney)
                .receivablesMoney(receivablesMoney)
                .receivablesSuccessMoney(receivablesSuccessMoney)
                .contractPer(contractPer)
                .receivablesPer(receivablesPer)
                .build();
    }
}
