package cn.iocoder.yudao.module.crm.service.crmanalysis;

import cn.hutool.core.util.NumberUtil;
import cn.iocoder.yudao.framework.common.enums.ShopCommonEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.crm.controller.admin.crmanalysis.vo.AchievementPageReqVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmanalysis.vo.CustomerLevelVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmanalysis.vo.UserRecordVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmanalysis.vo.UserVolumeVO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmcustomer.CrmCustomerDO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmrecord.CrmRecordDO;
import cn.iocoder.yudao.module.crm.dal.mysql.crmachievement.CrmAchievementMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmclues.CrmCluesMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmcontract.CrmContractMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmcontractreceivables.CrmContractReceivablesMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmcustomer.CrmCustomerMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmrecord.CrmRecordMapper;
import cn.iocoder.yudao.module.hrm.enums.TypesEnum;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.dict.dto.DictDataRespDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserApiVO;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserPageApiDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 客户分析 Service 实现类
 *
 * @author yshop
 */
@Service
@Validated
public class CustomerServiceImpl implements CustomerService {

    @Resource
    private CrmAchievementMapper achievementMapper;
    @Resource
    private DeptApi deptApi;
    @Resource
    private DictDataApi dictDataApi;
    @Resource
    private CrmContractMapper contractMapper;
    @Resource
    private CrmContractReceivablesMapper contractReceivablesMapper;
    @Resource
    private CrmCluesMapper crmCluesMapper;
    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private CrmCustomerMapper customerMapper;
    @Resource
    private CrmRecordMapper crmRecordMapper;

    @Override
    public CustomerLevelVO getCustomerLevel() {
        List<DictDataRespDTO> list = dictDataApi.getDictDataList("customer_level").getCheckedData();
        List<Long> allCustomer = new ArrayList<>();
        List<Long> dealCustomer = new ArrayList<>();

        list.forEach(v -> {
            LambdaQueryWrapper<CrmCustomerDO> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(CrmCustomerDO::getLevel, v.getValue());
            Long all = customerMapper.selectCount(wrapper);
            wrapper.eq(CrmCustomerDO::getDealStatus, ShopCommonEnum.IS_STATUS_1.getValue());
            Long deal = customerMapper.selectCount(wrapper);
            allCustomer.add(all);
            dealCustomer.add(deal);
        });
        List<String> data = list.stream().map(DictDataRespDTO::getLabel).collect(Collectors.toList());

        return CustomerLevelVO.builder().data(data).allCustomer(allCustomer).dealCustomer(dealCustomer).build();
    }

    @Override
    public CustomerLevelVO getCustomerSource() {
        List<DictDataRespDTO> list = dictDataApi.getDictDataList("customer_source").getCheckedData();

        List<Long> allCustomer = new ArrayList<>();
        List<Long> dealCustomer = new ArrayList<>();

        list.forEach(v -> {
            LambdaQueryWrapper<CrmCustomerDO> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(CrmCustomerDO::getSource, v.getValue());
            Long all = customerMapper.selectCount(wrapper);
            wrapper.eq(CrmCustomerDO::getDealStatus, ShopCommonEnum.IS_STATUS_1.getValue());
            Long deal = customerMapper.selectCount(wrapper);
            allCustomer.add(all);
            dealCustomer.add(deal);
        });
        List<String> data = list.stream().map(DictDataRespDTO::getLabel).collect(Collectors.toList());

        return CustomerLevelVO.builder().data(data).allCustomer(allCustomer).dealCustomer(dealCustomer).build();
    }


    @Override
    public CustomerLevelVO getCustomerIndustry() {
        List<DictDataRespDTO> list = dictDataApi.getDictDataList("customer_industry").getCheckedData();
        List<Long> allCustomer = new ArrayList<>();
        List<Long> dealCustomer = new ArrayList<>();

        list.forEach(v -> {
            LambdaQueryWrapper<CrmCustomerDO> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(CrmCustomerDO::getIndustry, v.getValue());
            Long all = customerMapper.selectCount(wrapper);
            wrapper.eq(CrmCustomerDO::getDealStatus, ShopCommonEnum.IS_STATUS_1.getValue());
            Long deal = customerMapper.selectCount(wrapper);
            allCustomer.add(all);
            dealCustomer.add(deal);
        });
        List<String> data = list.stream().map(DictDataRespDTO::getLabel).collect(Collectors.toList());

        return CustomerLevelVO.builder().data(data).allCustomer(allCustomer).dealCustomer(dealCustomer).build();
    }

    @Override
    public PageResult<UserVolumeVO> getCustomerVolume(AchievementPageReqVO pageReqVO) {
        AdminUserPageApiDTO dto = new AdminUserPageApiDTO();
        dto.setNickName(pageReqVO.getName());
        dto.setPageNo(pageReqVO.getPageNo());
        dto.setPageSize(pageReqVO.getPageSize());
        PageResult<AdminUserApiVO> pageResult = adminUserApi.getUserPage(dto).getCheckedData();
        PageResult<UserVolumeVO> pageResult1 = BeanUtils.toBean(pageResult, UserVolumeVO.class);
        pageResult1.getList().forEach(v -> {
            LambdaQueryWrapperX<CrmCustomerDO> wrapperX = new LambdaQueryWrapperX<>();
            wrapperX.eq(CrmCustomerDO::getOwnerUserId, v.getId())
                    .betweenIfPresent(CrmCustomerDO::getCreateTime, pageReqVO.getCreateTime());
            Long customerCount = customerMapper.selectCount(wrapperX);
            wrapperX.eq(CrmCustomerDO::getDealStatus, ShopCommonEnum.IS_STATUS_1.getValue());
            Long successCount = customerMapper.selectCount(wrapperX);

            v.setCustomerCount(customerCount);
            v.setSuccessCount(successCount);

            String per = "0";
            if (customerCount > 0) {
                per = NumberUtil.round(NumberUtil.div(successCount, customerCount), 2)
                        .multiply(new BigDecimal("100")).toString();
            }
            v.setSuccessPer(per);

        });

        return pageResult1;
    }

    @Override
    public PageResult<UserRecordVO> getRecord(AchievementPageReqVO pageReqVO) {
        AdminUserPageApiDTO dto = new AdminUserPageApiDTO();
        dto.setNickName(pageReqVO.getName());
        dto.setPageNo(pageReqVO.getPageNo());
        dto.setPageSize(pageReqVO.getPageSize());
        PageResult<AdminUserApiVO> pageResult = adminUserApi.getUserPage(dto).getCheckedData();
        PageResult<UserRecordVO> pageResult1 = BeanUtils.toBean(pageResult, UserRecordVO.class);
        pageResult1.getList().forEach(v -> {
            LambdaQueryWrapperX<CrmRecordDO> wrapperX = new LambdaQueryWrapperX<>();
            wrapperX.eq(CrmRecordDO::getCreator, v.getId())
                    .betweenIfPresent(CrmRecordDO::getCreateTime, pageReqVO.getCreateTime());
            Long totalCount = crmRecordMapper.selectCount(wrapperX);

            wrapperX.eq(CrmRecordDO::getTypes, TypesEnum.CUSTOMER.getValue());
            Long customerCount = crmRecordMapper.selectCount(wrapperX);

            wrapperX.eq(CrmRecordDO::getTypes, TypesEnum.BUSINESS.getValue());
            Long businessCount = crmRecordMapper.selectCount(wrapperX);

            wrapperX.eq(CrmRecordDO::getTypes, TypesEnum.CLUES.getValue());
            Long cluesCount = crmRecordMapper.selectCount(wrapperX);

            v.setTotalCount(totalCount);
            v.setCustomerCount(customerCount);
            v.setBusinessCount(businessCount);
            v.setCluesCount(cluesCount);

        });

        return pageResult1;
    }
}
