package cn.iocoder.yudao.module.crm.service.crmrecord;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.crm.controller.admin.crmrecord.vo.CrmRecordPageReqVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmrecord.vo.CrmRecordRespVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmrecord.vo.CrmRecordSaveReqVO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmbusiness.CrmBusinessDO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmclues.CrmCluesDO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmcustomer.CrmCustomerDO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmrecord.CrmRecordDO;
import cn.iocoder.yudao.module.crm.dal.mysql.crmbusiness.CrmBusinessMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmclues.CrmCluesMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmcustomer.CrmCustomerMapper;
import cn.iocoder.yudao.module.crm.dal.mysql.crmrecord.CrmRecordMapper;
import cn.iocoder.yudao.module.hrm.enums.RelationEnum;
import cn.iocoder.yudao.module.hrm.enums.TypesEnum;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.dict.dto.DictDataRespDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.hrm.enums.ErrorCodeConstants.RECORD_NOT_EXISTS;

/**
 * 跟进记录 Service 实现类
 *
 * @author yshop
 */
@Service
@Validated
public class CrmRecordServiceImpl implements CrmRecordService {

    @Resource
    private CrmRecordMapper recordMapper;
    @Resource
    private CrmCustomerMapper customerMapper;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private CrmBusinessMapper crmBusinessMapper;
    @Resource
    private CrmCluesMapper cluesMapper;
    @Resource
    private DictDataApi dictDataApi;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createRecord(CrmRecordSaveReqVO createReqVO) {
        // 插入
        CrmRecordDO record = BeanUtils.toBean(createReqVO, CrmRecordDO.class);
        recordMapper.insert(record);

        //更新客户
        if (TypesEnum.CUSTOMER.getValue().equals(createReqVO.getTypes())) {
            customerMapper.updateById(CrmCustomerDO.builder()
                    .id(createReqVO.getTypesId())
                    .nextTime(createReqVO.getNextTime())
                    .followStatus(createReqVO.getFollowStatus())
                    .followTime(LocalDateTime.now())
                    .build());
        } else if (TypesEnum.BUSINESS.getValue().equals(createReqVO.getTypes())) {
            //更新商机
            CrmBusinessDO crmBusinessDO = crmBusinessMapper.selectById(createReqVO.getTypesId());
            crmBusinessDO.setNextTime(createReqVO.getNextTime());
            crmBusinessMapper.updateById(crmBusinessDO);
            customerMapper.updateById(CrmCustomerDO.builder()
                    .id(crmBusinessDO.getCustomerId())
                    .nextTime(createReqVO.getNextTime())
                    .followTime(LocalDateTime.now())
                    .build());
        } else if (TypesEnum.CLUES.getValue().equals(createReqVO.getTypes())) {
            //更新线索
            cluesMapper.updateById(CrmCluesDO.builder()
                    .id(createReqVO.getTypesId())
                    .followStatus(createReqVO.getFollowStatus())
                    .nextTime(createReqVO.getNextTime())
                    .followTime(LocalDateTime.now())
                    .build());
        }


        // 返回
        return record.getId();
    }

    @Override
    public void updateRecord(CrmRecordSaveReqVO updateReqVO) {
        // 校验存在
        validateRecordExists(updateReqVO.getId());
        // 更新
        CrmRecordDO updateObj = BeanUtils.toBean(updateReqVO, CrmRecordDO.class);
        recordMapper.updateById(updateObj);
    }

    @Override
    public void deleteRecord(Long id) {
        // 校验存在
        validateRecordExists(id);
        // 删除
        recordMapper.deleteById(id);
    }

    private void validateRecordExists(Long id) {
        if (recordMapper.selectById(id) == null) {
            throw exception(RECORD_NOT_EXISTS);
        }
    }

    @Override
    public CrmRecordDO getRecord(Long id) {
        return recordMapper.selectById(id);
    }

    @Override
    public PageResult<CrmRecordRespVO> getRecordPage(CrmRecordPageReqVO pageReqVO) {
        List<Long> ids = new ArrayList<>();
        Long adminId = SecurityFrameworkUtils.getLoginUserId();
        if (RelationEnum.MY.getValue().equals(pageReqVO.getRelation())) {
            ids.add(adminId);
        } else if (RelationEnum.SUB.getValue().equals(pageReqVO.getRelation())) {
            ids = adminUserApi.getUserListBySubordinateIds(adminId).getCheckedData();
        }
        PageResult<CrmRecordDO> pageResult = recordMapper.selectPage(pageReqVO, ids);
        PageResult<CrmRecordRespVO> pageResult1 = BeanUtils.toBean(pageResult, CrmRecordRespVO.class);


        for (CrmRecordRespVO crmRecordRespVO : pageResult1.getList()) {
            DictDataRespDTO dto = dictDataApi.getDictData("follow_status", crmRecordRespVO.getRecordType().toString()).getCheckedData();
            crmRecordRespVO.setRecordTypeName(dto.getLabel());
            if (TypesEnum.CUSTOMER.getValue().equals(crmRecordRespVO.getTypes())) {
                CrmCustomerDO customerDO = customerMapper.selectById(crmRecordRespVO.getTypesId());
                crmRecordRespVO.setTypesName(customerDO.getName());
            } else if (TypesEnum.BUSINESS.getValue().equals(crmRecordRespVO.getTypes())) {
                CrmBusinessDO crmBusinessDO = crmBusinessMapper.selectById(crmRecordRespVO.getTypesId());
                crmRecordRespVO.setTypesName(crmBusinessDO.getName());
            } else if (TypesEnum.CLUES.getValue().equals(crmRecordRespVO.getTypes())) {
                CrmCluesDO crmCluesDO = cluesMapper.selectById(crmRecordRespVO.getTypesId());
                crmRecordRespVO.setTypesName(crmCluesDO.getName());
            }
            AdminUserRespDTO adminUserRespDTO = adminUserApi.getUser(crmRecordRespVO.getCreator()).getCheckedData();
            crmRecordRespVO.setOwnUserName(adminUserRespDTO.getNickname());

        }
        return pageResult1;
    }

}
