package cn.iocoder.yudao.module.system.controller.admin.rental;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.customer.RentalCustomerPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.customer.RentalCustomerRespVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.customer.RentalCustomerSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.customer.RentalCustomerStatusReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.rental.RentalCustomerDO;
import cn.iocoder.yudao.module.system.service.rental.RentalCustomerService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;


@Tag(name = "管理后台 - 租赁客户")
@RestController
@RequestMapping("/system/rental-customer")
@Validated
public class RentalCustomerController {

    @Resource
    private RentalCustomerService rentalCustomerService;

    @PostMapping("/create")
    @Operation(summary = "创建租赁客户")
    @PreAuthorize("@ss.hasPermission('system:rental-customer:create')")
    public CommonResult<Long> createRentalCustomer(@Valid @RequestBody RentalCustomerSaveReqVO createReqVO) {
        return success(rentalCustomerService.createRentalCustomer(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新租赁客户")
    @PreAuthorize("@ss.hasPermission('system:rental-customer:update')")
    public CommonResult<Boolean> updateRentalCustomer(@Valid @RequestBody RentalCustomerSaveReqVO updateReqVO) {
        rentalCustomerService.updateRentalCustomer(updateReqVO);
        return success(true);
    }

    @PutMapping("/update-status")
    @Operation(summary = "更新租赁客户状态")
    @PreAuthorize("@ss.hasPermission('system:rental-customer:update')")
    public CommonResult<Boolean> updateRentalCustomer(@RequestBody RentalCustomerStatusReqVO updateReqVO) {
        rentalCustomerService.updateRentalCustomerStatus(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除租赁客户")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('system:rental-customer:delete')")
    public CommonResult<Boolean> deleteRentalCustomer(@RequestParam("id") Long id) {
        rentalCustomerService.deleteRentalCustomer(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得租赁客户")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:rental-customer:query')")
    public CommonResult<RentalCustomerRespVO> getRentalCustomer(@RequestParam("id") Long id) {
        RentalCustomerDO rentalCustomer = rentalCustomerService.getRentalCustomer(id);
        return success(BeanUtils.toBean(rentalCustomer, RentalCustomerRespVO.class));
    }

    @GetMapping("/get-list")
    @Operation(summary = "获得开启状态的租赁客户列表")
    @PreAuthorize("@ss.hasPermission('system:rental-customer:query')")
    public CommonResult<List<RentalCustomerRespVO>> getRentalCustomerList() {
        List<RentalCustomerDO> rentalCustomer = rentalCustomerService.getRentalCustomerList();
        return success(BeanUtils.toBean(rentalCustomer, RentalCustomerRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得租赁客户分页")
    @PreAuthorize("@ss.hasPermission('system:rental-customer:query')")
    public CommonResult<PageResult<RentalCustomerRespVO>> getRentalCustomerPage(@Valid RentalCustomerPageReqVO pageReqVO) {
        PageResult<RentalCustomerDO> pageResult = rentalCustomerService.getRentalCustomerPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, RentalCustomerRespVO.class));
    }
}